# frozen_string_literal: true

module RailsRbs
  # This controller class is the C.R.U.D. interface for rule-sets.  Whenever the RailsRbs configuration
  # uses rule_owners (@see RailsRbs Config) rule_sets will only be displayed and modified that belong
  # to that owner.  This controller will attempt to render views under a rule_sets/ directory if view
  # rendering is enabled (again @see RailsRbs Config)
  #
  # @note When locating a rule_set by ID this controller will use the config setting - request_id_field
  #   like all other configuration @see RailsRbs Config
  class RuleSetsController < RailsRbs.parent_controller.constantize
    include RailsRbs::Controllers::Helpers
    # If owner access is enabled validate that an owner is present
    before_action :validate_owner_access
    before_action :create_rule_set, only: :create
    before_action :load_rule_set, only: [:show, :update, :destroy]

    # #index displays all rule_sets either for the owner or from the entire table if owner_access is
    # disabled (@see RailsRbs Config).  The rule_sets will always include any associated rules.
    def index
      if RailsRbs.owner_access
        @rule_sets = rule_owner.rule_sets.includes(:rules, :rule_actions)
      else
        @rule_sets = RuleSet.all.includes(:rules, :rule_actions)
      end
      render_view_fallback('rule_sets/index') do
        render json: @rule_sets
      end
    end
    # #show displays a single rule_set depending on the provided id.  The method uses a
    # protected load method that will raise an ActiveRecord::RecordNotFound error if a rule_set is
    # not located.  This includes providing an existing ID for a rule_set not owned by the
    # requesting owner (only if owner access is enabled - @see RailsRbs Config).
    def show
      render_view_fallback('rule_sets/show') do
        render json: @rule_set
      end
    end
    # #update updates a single rule_set, including any associated rules by first using the
    # protected load_rule_set method (@see #show).  If an update fails the action will raise
    # an ActiveRecord::RecordInvalid error.  Similar to #show this method will only operate on
    # rule_sets that belong to the rule_set owner (if owner access is enabled).
    def update
      update_with_nested(key: :rules_attributes, klass: Rule)
      update_with_nested(key: :rule_actions_attributes, klass: RuleAction)
      @rule_set.update!(rule_set_params) unless rule_set_params.empty? # This can be empty after update_with_nested
      # Force a reload in case the above update never ran
      @rule_set.reload
      render_view_fallback('rule_sets/show') do
        render json: @rule_set
      end
    end
    # #create will attempt to create a new rule_set and like all other actions in this
    # controller it will do so for the requesting owner if owner access is enabled
    # (@see RailsRbs Config).  If owner access is enabled but no requesting owner can be found
    # an ActiveRecord::RecordNotFound error will be raised.
    def create
      render_view_fallback('rule_sets/show') do
        render json: @rule_set
      end
    end
    # #destroy will attempt to destroy a rule_set loading it using the same protected load rule_set
    # used by #show and #update.  If destroy causes any issues an ActiveRecord error will be raised,
    # otherwise the action renders the index view or nothing at all if view rendering is disabled
    # (@see RailsRbs Config)
    def destroy
      @rule_set.destroy!
      render_view_fallback('rule_sets/index') do
        render nothing: true, status: 200
      end
    end

    protected
    # #load_rule_set is meant to be used as a before action to ensure the appropriate resource is
    # located for whatever crud operation is interested.  In this case the method will raise an
    # ActiveRecord::RecordNotFound error when no record is found.
    def load_rule_set
      if RailsRbs.owner_access
        rule_sets_association = rule_owner.rule_sets
      else
        rule_sets_association = RuleSet
      end
      @rule_set = rule_sets_association.includes(:rules, :rule_actions).find_by!(RailsRbs.request_id_field => params[:id])
    end

    # create_rule_set is meant to be used as a before action that needs to instantiate a new
    # rule_set resource.  This method like all others operates on the rule_owners rule_sets if
    # owner access is enabled.
    def create_rule_set
      if RailsRbs.owner_access
        @rule_set = rule_owner.rule_sets.create!(rule_set_params)
      else
        @rule_set = RuleSet.create!(rule_set_params)
      end
      raise ActiveRecord::InvalidRecord.new(@rule_set) if @rule_set.invalid?
      @rule_set
    end

    def rule_set_params
      return @rule_set_params if @rule_set_params.present?
      rule_fields = RailsRbs.base_rule_fields.concat(RailsRbs.rule_fields).concat([:id, :_destroy]).uniq.reject(&:nil?)
      rule_set_fields = [:scope, {rules_attributes: rule_fields, rule_actions_attributes: [RailsRbs.request_id_field.to_sym, :id, :_destroy, :enforced_value, :enforced_field]}].reject(&:nil?)
      rule_set_fields << "#{RailsRbs.owned_by}_id".to_sym if RailsRbs.owner_access
      @rule_set_params = params.require(RailsRbs.rule_set_params_root.to_sym).permit(rule_set_fields)
    end

    def validate_owner_access
      if RailsRbs.owner_access && rule_owner.nil?
        raise ActiveRecord::RecordNotFound.new("Unable to find owner required to create rule_set")
      end
    end

    def update_with_nested(key: ,klass:)
      # If using id as the rule set identifier let accepts_nested_attributes do it's job
      if RailsRbs.request_id_field.to_sym != :id && rule_set_params[key].present?
        id_field = RailsRbs.request_id_field.to_sym
        updates, deletes, creates = [], [], []
        rule_set_params[key].each do |attrs|
          attrs[:rule_set_id] ||= @rule_set.try(:id)
          if attrs[:id].present?
            if attrs[:_destroy]
              deletes << attrs
            else
              updates << attrs
            end
          else
            creates << attrs
          end
        end
        updates.each { |attrs| klass.find_by!(id_field => attrs.delete('id')).update(attrs) }
        klass.where(id_field => deletes.map { |attrs| attrs['id'] }).delete_all
        creates.each { |attrs| klass.create!(attrs) }
        rule_set_params[key] = {}
      end
    end

    ActiveSupport.run_load_hooks(:rule_sets_controller, self)
  end
end
