# frozen_string_literal: true

module RailsRbs
  # This controller class is the C.R.UD. interface for rules.  Whenever the RailsRbs configuration
  # uses rule_owners (@see RailsRbs Config) rules will only be displayed and modified that belong
  # to that owner.  This controller will attempt to render views under a rules/ directory if view
  # rendering is enabled ( again @see RailsRbs Config)
  #
  # @note when locationg a rule by ID this contoller will use the config settings - request_id_field
  # like all other configuations @see RailsRbs Config
  class RulesController < RailsRbs.parent_controller.constantize
    include RailsRbs::Controllers::Helpers
    # If owner access is enabled validate that an owner is present
    before_action :validate_owner_access
    before_action :load_rule, only: [:show, :update, :destroy]
    before_action :create_rule, only: :create

    # #index displays all rules either for the owner or from the entire table if owner_access is
    # disabled (@see RailsRbs Config).
    def index
      if RailsRbs.owner_access
        @rules = rule_owner.try(:rules)
      else
        @rules = Rule.all
      end
      render_view_fallback('rules/index') do
        render json: @rules
      end
    end
    # #show displays a single rule depending on the provided id.  The method uses a protected
    # load method that will rails an ActiveSupport::RecordNotFound error if the rule is not
    # located.  This includes providing an ID for an existing rule not owned by the requesting
    # owner (if owner access is enabled - @see RailsRbs Config).
    def show
      render_view_fallback('rules/show') do
        render json: @rule
      end
    end
    #update updates a single rule by first using a protected load method to locate the rule
    # (@see #show).  If an update fails the action will rails an ActiveRecord::RecordInvalid
    # error.  Similar to #show this method will only operate on rules that belong to the rule
    # owner if owner access is enabled.
    def update
      @rule.update!(rule_params)
      render_view_fallback('rules/show') do
        render json: @rule
      end
    end
    # #create will attempt to create a new rule and like the other actions in this controller
    # it will do so for the requesting owner if owner access is enabled (@see RailsRbs Config).
    # If owner_access is enabled but no requesting owner can be found an
    # ActiveRecord::RecordNotFound error will be raised.
    def create
      render_view_fallback('rules/show') do
        render json: @rule
      end
    end
    # #destroy will attempt to destroy a rule loading it using the same protected load rule method
    # used by #show and #update.  If destroy causes any issues an ActiveRecord error will be raised,
    # otherwise the action renders the index view or nothing at lal if view rendering is disabled
    # (@see RailsRbs Config).
    def destroy
      @rule.destroy!
      render_view_fallback('rules/index') do
        render nothing: true, status: 200
      end
    end

    protected
    # #load_rule is meant to be used as a before action to ensure the appropriate resource is
    # located for whatever crud operation is interested.  In this case the method will raise an
    # ActiveRecord::RecordNotFound error when no record is found.
    def load_rule
      if RailsRbs.owner_access
        rules_association = rule_owner.rules
      else
        rules_association = Rule
      end
      @rule = rules_association.find_by!(RailsRbs.request_id_field => params[:id])
    end

    # Attempt to fetch the rule_set for the provided parameters and fall back to any rule object
    # that may be in context for this request.
    def rule_set
      if RailsRbs.owner_access
        @rule_set ||= rule_owner.rule_sets.find_by!(id: rule_params[:rule_set_id])
      else
        @rule_set ||= RuleSet.find_by!(id: rule_params[:rule_set_id])
      end
      @rule_set ||= @rule.try(:rule_set)
    end

    def create_rule
      if RailsRbs.owner_access
        @rule = rule_owner.rules.create!(rule_params)
      else
        @rule = Rule.create!(rule_params)
      end
    end

    def rule_params
      rule_fields = RailsRbs.base_rule_fields.concat(RailsRbs.rule_fields).uniq.reject(&:nil?)
      params.require(RailsRbs.rule_params_root.to_sym).permit(rule_fields)
    end

    def validate_owner_access
      if RailsRbs.owner_access && rule_owner.nil?
        raise ActiveRecord::RecordNotFound.new("Unable to find owner required to create rule")
      end
    end

    ActiveSupport.run_load_hooks(:rules_controller, self)
  end
end
